/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.domain.permission;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import java.io.Serializable;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.annotaions.Comment;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.snowflake.SnowflakeId;

/**
 * 权限
 *
 * @author ifzm
 * 2018/9/3 17:08
 */

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Comment("系统权限")
@Table(
    name = "SYS_PERMISSION",
    indexes = {
        @Index(columnList = "type"),
        @Index(columnList = "pid"),
        @Index(columnList = "tree"),
        @Index(columnList = "uid", unique = true),
    }
)
public class Permission implements Serializable {

    private static final long serialVersionUID = -2001221843529000953L;

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.LONG)
    Long id;

    @Comment("类型(MENU: 菜单, BUTTON: 按钮)")
    @Enumerated(EnumType.STRING)
    Type type;

    @Comment("序号")
    Integer serial;

    @Comment("标识")
    @Column(unique = true, nullable = false)
    String uid;

    @Comment("名称")
    String name;

    @Comment("路径")
    String path;

    @Comment("重定向")
    String redirect;

    @Comment("组件")
    String component;

    @Comment("图标")
    String icon;

    @Comment("是否隐藏")
    Boolean hidden;

    @Comment("是否启用")
    Boolean enable;

    @Comment("权限父级ID")
    Long pid;

    @Comment("树节点标识")
    String tree;

    /**
     * 可用于扩展一些前端可能用到的路由参数
     */
    @Comment("扩展元数据信息")
    @Lob
    @Column(columnDefinition = "JSON")
    String meta;

    @SuppressWarnings({ "unused" })
    public enum Type {
        /**
         * 目录/平台
         */
        DIR,

        /**
         * 菜单
         */
        MENU,

        /**
         * 按钮
         */
        BUTTON,
    }

    /**
     * 获取权限的元数据信息，通常是根据前端所需来输出，可自定义调整
     * @return JSON 格式 Meta 元数据
     */
    public JSONObject getMetaJson() {
        JSONObject meta = new JSONObject();
        meta.put("title", this.name);
        meta.put("icon", this.icon);
        meta.put("orderNo", this.serial);
        meta.put("hideMenu", this.hidden);

        try {
            String raw = this.meta.replace("\\", "").replaceAll("^\"(.*)\"$", "$1");
            meta.putAll(JSON.parseObject(raw));
        } catch (Exception ignored) {}

        return meta;
    }
}
